<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class WV_Filter {
	public static function filterArticles(WV_Filter_Interface $filter, Closure $comparator = null) {
		$where = self::optimizeArticleQuery($filter);
		$values = sly_Core::getContainer()->getArticleService()->find($where);

		return self::filter($values, $filter, $comparator);
	}

	public static function filterCategories(WV_Filter_Interface $filter, Closure $comparator = null) {
		$values = sly_Core::getContainer()->getCategoryService()->find();

		return self::filter($values, $filter, $comparator);
	}

	public static function filterMedia(WV_Filter_Interface $filter, Closure $comparator = null) {
		$values = sly_Core::getContainer()->getMediumService()->find();

		return self::filter($values, $filter, $comparator);
	}

	public static function filterUser(WV_Filter_Interface $filter, Closure $comparator = null) {
		$values = sly_Core::getContainer()->getUserService()->find();

		return self::filter($values, $filter, $comparator);
	}

	public static function filter($values, WV_Filter_Interface $filter, Closure $comparator = null) {
		foreach ($values as $idx => $value) {
			if ($filter->isValueFiltered($value)) {
				unset($values[$idx]);
			}
		}

		if (is_callable($comparator)) {
			usort($values, $comparator);
		}

		return $values;
	}

	private static function optimizeArticleQuery($filter) {
		$where = array();
		$typeFilter = self::findInsideAnd($filter, 'WV_Filter_Article_Type');
		if ($typeFilter && $typeFilter instanceof WV_Filter_Article_Type) {
			$where['type'] = $typeFilter->getType();
		}
		return $where;
	}

	private static function findInsideAnd($filter, $class) {
		if ($filter instanceof $class) {
			return $filter;
		}
		if ($filter instanceof WV_Filter_And) {
			foreach ($filter->getFilters() as $filter_i) {
				$found = self::findInsideAnd($filter_i, $class);
				if ($found) {
					return $found;
				}
			}
		}
		return null;
	}
}
