<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Listener;

use sly_Core;
use sly_Model_Language;
use wv\metainfo\Metainfo;

class Language {
	/**
	 * Handler for SLY_CLANG_ADDED
	 *
	 * copies all infos of the default language.
	 *
	 * @param sly_Model_Language $language  the added language
	 * @param array              $params    additional parameters
	 */
	public function onLanguageAdded(sly_Model_Language $language, array $params) {
		$langs = \sly_Util_Language::findAll(true);

		// do nothing if the first language was added
		if (count($langs) === 1) {
			return;
		}

		$defaultLang = sly_Core::getDefaultClangId();
		if (!in_array($defaultLang, $langs)) {
			$defaultLang = reset($langs);
		}

		$container = Metainfo::getContainer();
		$service   = $container['metainfo-service-value'];

		$service->copy(array('clang' => $defaultLang), array('clang' => $language->getId()), null);

		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_CLANG_DELETED
	 *
	 * Deletes all Metainfos for a language
	 *
	 * @param sly_Model_Language $language  the deleted language
	 * @param array              $params    additional parameters
	 */
	public function onLanguageDeleted(sly_Model_Language $language, array $params) {
		$container = Metainfo::getContainer();
		$service   = $container['metainfo-service-value'];

		$service->deleteClang($language->getId());

		Metainfo::clearDataCache();
	}
}
