<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2\Cache;

use wv\realurl2\URL;

/**
 * Cache-Wrapper
 *
 * @author Christoph
 * @since  v4.0
 */
abstract class Persistent {
	/**
	 * get system cache
	 *
	 * @return wv\BabelCache\CacheInterface
	 */
	protected static function getCache() {
		return \sly_Core::getContainer()->get('sly-cache');
	}

	public static function findURL($namespace, $ident) {
		return self::getCache()->get('realurl2.ns.'.$namespace, $ident);
	}

	public static function clear($namespace = null, $ident = null) {
		$cacheNS = 'realurl2'.($namespace === null ? '' : '.ns.'.$namespace);
		$cache   = self::getCache();

		if ($ident !== null) {
			$cache->delete($cacheNS, $ident);
		}
		else {
			$cache->clear($cacheNS, true);
		}
	}

	public static function cacheURL(URL $url) {
		$cacheNS  = 'realurl2.ns.'.$url->getNamespace();
		$cacheKey = $url->getIdent();

		self::getCache()->set($cacheNS, $cacheKey, $url);

		// Do NOT set the URL based on its URI's hash here, as this might
		// overwrite another article's URL with higher priority!
//		self::getCache()->set('realurl2.volatile', md5($url->getRawURL()), $url);
	}
}
